"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const shared_utils_1 = require("@vuepress/shared-utils");
/**
 * Handle options from users.
 * @param options
 * @param ctx
 * @return {*}
 */
function handleOptions(options, ctx) {
    let { directories = [] } = options;
    /**
     * Validate the existence of directory specified by directory classifier.
     */
    directories = directories.filter(({ dirname }) => {
        const targetDir = path_1.join(ctx.sourceDir, dirname);
        if (fs_1.existsSync(targetDir)) {
            return true;
        }
        shared_utils_1.logger.warn(`[plugin-directory-classifier]` +
            `  Invalid directory classifier: ${shared_utils_1.chalk.cyan(dirname)}, ` +
            `  ${shared_utils_1.chalk.gray(targetDir)} doesn't exist!}`);
        return false;
    });
    const indexPages = [];
    const pageEnhancers = [];
    /**
     * 1. Directory-based classification
     */
    for (const directory of directories) {
        const { dirname, layout: indexLayout = "IndexPage", frontmatter, itemLayout = "Page", subdirlevel = 1 } = directory;
        const indexPath = `/${dirname}/`;
        /**
         * 1.1 Required index path.
         */
        if (!dirname || !indexPath) {
            continue;
        }
        /**
         * 1.2 Inject index page.
         */
        indexPages.push({
            permalink: indexPath,
            frontmatter: {
                layout: ctx.getLayout(indexLayout),
                subdirlevel,
                indexed: [],
                ...frontmatter
            },
            meta: {
                id: dirname
            }
        });
        /**
         * 1.3 Set layout for pages that match current directory classifier.
         */
        pageEnhancers.push({
            when: ({ regularPath }) => isIndexed(regularPath, indexPath, subdirlevel),
            frontmatter: {
                layout: itemLayout || "Page"
            },
            data: {
                id: dirname
            }
        });
    }
    return {
        pageEnhancers,
        indexPages
    };
}
exports.handleOptions = handleOptions;
/**
 * Filter pages in `dirname`, i.e. only top-level files or index files in sub directories.
 * @param regularPath
 * @param indexPath
 * @param level
 * @return {*}
 */
function isIndexed(indexedPath, indexPath, level) {
    return (Boolean(indexedPath) &&
        indexedPath !== indexPath &&
        indexedPath.startsWith(indexPath) &&
        (indexedPath.endsWith("index.html") ||
            indexedPath.endsWith("/") ||
            isWithinSubDir(indexPath, indexedPath, level)));
}
exports.isIndexed = isIndexed;
/**
 * Subtract `sub` from `text`.
 * @param text
 * @param sub
 * @return {*}
 */
function diff(text, sub) {
    return text.split(sub).join("");
}
/**
 * Test weather subtraction makes sense.
 * @param text
 * @param sub
 * @return {*}
 */
function diffValidate(text, sub) {
    return text !== diff(text, sub);
}
/**
 * Test if path is within the next`level` sub directories.
 * @param index
 * @param path
 * @param level
 * @return {*}
 */
function isWithinSubDir(index, path, level = 1) {
    if (level > 0 && diffValidate(path, index)) {
        const dirLevel = diff(path, index).split("/").length;
        return dirLevel > 0 && dirLevel <= level + 1;
    }
    return false;
}
